/*
Yahoo-Transport parser.c Handles data from etherx
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <yahootrans.h>

/* incoming iq get */
void yahoo_parse_iq_get(yahootrans_jpacket yjp)
{
    jpacket jp = yjp->jp;
    xmlnode query;
    yahoo_session s;

    jutil_iqresult(jp->x);
    query = xmlnode_get_tag(jp->x, "query");
    if(query == NULL) 
        query = xmlnode_insert_tag(jp->x, "query");

    s = yahoo_find_session(yjp->yti, jp->from);

    if(jp->to->user == NULL) /* iq get to a server */
    {
        if(NSCHECK(jp->iq, NS_REGISTER))
        {
            xmlnode x = yahoo_xdb_get(yjp->yti, jp->to->server, jp->from);
            xmlnode_put_attrib(query,"xmlns",NS_REGISTER);
            xmlnode_insert_tag(query, "username");
            xmlnode_insert_tag(query, "password");
            xmlnode_insert_cdata(xmlnode_insert_tag(query, "key"), jutil_regkey(NULL, jid_full(jp->to)), -1);
            xmlnode_insert_cdata(xmlnode_insert_tag(query, "instructions"), "Enter your YAHOO! Messenger Username and Password.", -1);
            if(x != NULL) 
            { /* we have XDB data for this user */
                xmlnode_insert_tag(query, "registered");
                xmlnode_insert_cdata(xmlnode_get_tag(query, "username"), xmlnode_get_attrib(x, "id"), -1);
                xmlnode_free(x);
            }
        }
        else if(NSCHECK(jp->iq, NS_VERSION))
        {
            xmlnode_put_attrib(query, "xmlns", NS_VERSION);
            xmlnode_insert_cdata(xmlnode_insert_tag(query, "name"), "YAHOO! Messenger", -1);
            xmlnode_insert_cdata(xmlnode_insert_tag(query, "ver"), YAHOO_TRANS_VERSION, -1);
        }
        else if(NSCHECK(jp->iq, NS_TIME))
        {
            xmlnode_put_attrib(query, "xmlns", NS_TIME);
            xmlnode_insert_cdata(xmlnode_insert_tag(query, "utc"), jutil_timestamp(), -1);

            xmlnode_insert_cdata(xmlnode_insert_tag(query, "tz"), "gmt", -1);
        }
        else if(NSCHECK(jp->iq, NS_LAST))
        {
            xmlnode_put_attrib(query, "xmlns", NS_LAST);
            xmlnode_put_attrib(query, "stamp", yjp->yti->start_time);
        }
        else if(NSCHECK(jp->iq, NS_VCARD))
        {
            xmlnode_put_attrib(query, "xmlns", NS_VCARD);
            xmlnode_insert_node(query, xmlnode_get_firstchild(xmlnode_get_tag(yjp->yti->c, "vcard")));
        }
        else if(NSCHECK(jp->iq, NS_GATEWAY))
        {
            xmlnode gateway = xmlnode_get_tag(yjp->yti->c, "gateway");
            xmlnode prompt  = xmlnode_get_tag(jp->iq, "prompt");
            xmlnode_put_attrib(query, "xmlns", NS_GATEWAY);

            if(prompt != NULL)
            {
                char *username = xmlnode_get_data(prompt);
                if(username == NULL)
                {
                    jutil_error(jp->x, TERROR_NOTACCEPTABLE);
                    jutil_tofrom(jp->x);
                    deliver(dpacket_new(jp->x), yjp->yti->i);
                    return;
                }
                xmlnode_insert_cdata(xmlnode_insert_tag(query, "jid"), spools(jp->p, username, "@", jp->to->server, jp->p), -1);
            }
            else
            {
                if(xmlnode_get_tag(gateway, "desc") == NULL)
                    xmlnode_insert_cdata(xmlnode_insert_tag(query, "desc"), "Please enter the Yahoo ID of the person you wish to contact", -1);
                else
                    xmlnode_insert_cdata(xmlnode_insert_tag(query, "desc"), xmlnode_get_tag_data(gateway, "desc"), -1);
    
                if(xmlnode_get_tag(gateway, "prompt") == NULL)
                    xmlnode_insert_cdata(xmlnode_insert_tag(query, "prompt"), "Yahoo ID", -1);
                else
                    xmlnode_insert_cdata(xmlnode_insert_tag(query, "prompt"), xmlnode_get_tag_data(gateway, "prompt"), -1);
            }
        }
        else if(NSCHECK(jp->iq, NS_BROWSE))
        {
            xmlnode service;
            xmlnode_put_attrib(query, "xmlns", NS_BROWSE);
            service = xmlnode_insert_tag(query, "service");
            xmlnode_put_attrib(service, "type", "yahoo");
            xmlnode_put_attrib(service, "jid", jid_full(yjp->jp->to));
            xmlnode_put_attrib(service, "name", "Yahoo! Transport Service");
            xmlnode_insert_cdata(xmlnode_insert_tag(service, "ns"), NS_BROWSE, -1);
            xmlnode_insert_cdata(xmlnode_insert_tag(service, "ns"), NS_GATEWAY, -1);
            xmlnode_insert_cdata(xmlnode_insert_tag(service, "ns"), NS_REGISTER, -1);
            xmlnode_insert_cdata(xmlnode_insert_tag(service, "ns"), NS_VCARD, -1);
            xmlnode_insert_cdata(xmlnode_insert_tag(service, "ns"), NS_LAST, -1);
            xmlnode_insert_cdata(xmlnode_insert_tag(service, "ns"), NS_TIME, -1);
            xmlnode_insert_cdata(xmlnode_insert_tag(service, "ns"), NS_VERSION, -1);
        }
        else
        {
            jutil_error(jp->x, TERROR_NOTIMPL);
            jutil_tofrom(jp->x);
        }
    }
    else /* iq get to a user */
    {
        if(NSCHECK(jp->iq, NS_BROWSE))
        {
            xmlnode service;
            xmlnode_put_attrib(query, "xmlns", NS_BROWSE);
            service = xmlnode_insert_tag(query, "user");
            xmlnode_put_attrib(service, "type", "client");
            xmlnode_put_attrib(service, "jid", jid_full(yjp->jp->to));
            xmlnode_put_attrib(service, "name", jp->to->user);
            xmlnode_insert_cdata(xmlnode_insert_tag(service, "ns"), NS_BROWSE, -1);
        }
        else
        {
            jutil_error(jp->x, TERROR_NOTIMPL);
            jutil_tofrom(jp->x);
        }
    }

    deliver(dpacket_new(jp->x), yjp->yti->i);
}

void yahoo_parse_iq_set(yahootrans_jpacket yjp)
{
    jpacket jp = yjp->jp;
    yahoo_session s;
    xmlnode query;

    s = yahoo_find_session(yjp->yti, jp->from);

    if(jp->to->user == NULL) /* iq set to the transport */
    {
        if(NSCHECK(jp->iq,NS_REGISTER))
        {
            char *username;
            char *password;
    
            query = xmlnode_get_tag(jp->x, "query?xmlns=jabber:iq:register");
            username = xmlnode_get_tag_data(query, "username");
            password = xmlnode_get_tag_data(query, "password");
    
            if(username == NULL || password == NULL)
            {
                yahoo_send_error(yjp->yti, jp->from, "406", "You have to send both a password AND a username to register with Yahoo! Transport");
                jutil_iqresult(jp->x);
                deliver(dpacket_new(jp->x), yjp->yti->i); 
                return;
            }
    
            if(s != NULL)
            { /* if they have an existing session, let's kill this subscription, and add it back later */
                yahoo_send_presence(s, NULL, jid_full(s->id), "unsubscribe", "Removing Subscription", NULL);
                yahoo_send_presence(s, NULL, jid_full(s->id), "unsubscribed", "Removing Subscription", NULL);
                yahoo_session_end(s);
            }
    
            log_debug("yahoo", "[YAHOO] registration received, attempting to log in %s as yahoo user: %s", jid_full(jp->from), username);
    
            if(yahoo_session_create(yjp->yti, username, password, jp) != NULL)
            {
                xmlnode logon = xmlnode_new_tag("logon");
                xmlnode_put_attrib(logon, "id", username);
                xmlnode_put_attrib(logon, "pass", password);
    
                yahoo_xdb_set(yjp->yti, jp->to->server, jp->from, logon);
            }
    
            jutil_iqresult(jp->x);
            deliver(dpacket_new(jp->x), yjp->yti->i);
        }
        else
        {
            
            jutil_error(jp->x, TERROR_NOTIMPL);
            deliver(dpacket_new(jp->x), yjp->yti->i);
        }
    }
    else /* iq set to a user */
    {
        if(NSCHECK(jp->iq,NS_OOB))
        {
            char *url, *desc;
            spool sp;
    
            if(s == NULL || s->Con == NULL)
            {
                xmlnode_free(jp->x);
                return;
            }
            url = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(xmlnode_get_tag(jp->x, "query"), "url")));
            desc = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(xmlnode_get_tag(jp->x, "query"), "desc")));

            log_debug("yahoo", "[YAHOO] sendind url (%s) to user (%s) activeID (%s)", url, jp->to->user, s->Con->user);

            sp = spool_new(jp->p);
            spooler(sp, s->Con->user, ", (who is using jabber), would like to send you a file.  This file (", desc,") is located at: ", url," ... Due to the limitations of Jabber and Yahoo, this file could not be sent directly, but will be implemented at a later date.  Sorry for the inconvenience", sp);

            yahoo_cmd_msg_offline(s->Con, s->Con->login_id, jp->to->user, spool_print(sp));
            xmlnode_free(jp->x);
        }
        else
        {
            
            jutil_error(jp->x, TERROR_NOTIMPL);
            deliver(dpacket_new(jp->x), yjp->yti->i);
        }
    }
}

/* incoming packets from etherx get parsed here */
void yahoo_parse_jpacket(void *arg)
{
    yahootrans_jpacket yjp = (yahootrans_jpacket)arg;
        
    if(yjp == NULL || yjp->jp == NULL) 
        return;

    switch(yjp->jp->type)
    {
    case JPACKET_IQ:
        if(jpacket_subtype(yjp->jp) == JPACKET__GET)
            yahoo_parse_iq_get(yjp);
        else if(jpacket_subtype(yjp->jp) == JPACKET__SET)
            yahoo_parse_iq_set(yjp);
        else
            xmlnode_free(yjp->jp->x);
        break;
    case JPACKET_MESSAGE:
        if(jpacket_subtype(yjp->jp) == JPACKET__ERROR)
            xmlnode_free(yjp->jp->x);
        else
            yahoo_parse_message(yjp);
        break;
    case JPACKET_S10N:
        switch(jpacket_subtype(yjp->jp))
        {
        case JPACKET__SUBSCRIBE:
            yahoo_subscribe_user(yjp);
            break;
        case JPACKET__UNSUBSCRIBE:
            yahoo_unsubscribe_user(yjp);
            break;
        default:
            xmlnode_free(yjp->jp->x);
            break;
        }
        break;
    case JPACKET_PRESENCE:
        yahoo_parse_presence(yjp);
        break;
    default:
        xmlnode_free(yjp->jp->x);
    }
    return;
}

