#include <yahootrans.h>

void yahoo_auth_user(yahootrans_jpacket yjp)
{
    jpacket jp = yjp->jp;
    xmlnode x;
    yahoo_session s;
    char *username, *password;


    s = yahoo_find_session(yjp->yti, jp->from);
    if(s != NULL) log_debug("yahoo", "auth belongs to session %X (so not starting a new one)", s);

    if(s != NULL)
        return; /* session already exists */

    x = yahoo_xdb_get(yjp->yti, jp->to->server, jp->from);

    if(x == NULL)
    { /* no data for this user */

        log_debug("yahoo","no XDB checking xres...");
        /* last chance, check for xres stuff */
        x = jid_xres(jp->to);

        username = xmlnode_get_attrib(x, "username");
        password = xmlnode_get_attrib(x, "pass");

        if(username != NULL && password != NULL)
        { /* good xres found */
            xmlnode message, body, subject;
            xmlnode yahoodata;

            log_debug("yahoo", "good xres found, setting XDB to this");
            /* set XDB */
            yahoodata = xmlnode_new_tag("logon");
            xmlnode_put_attrib(yahoodata, "id", username);
            xmlnode_put_attrib(yahoodata, "pass", password);

            yahoo_xdb_set(yjp->yti, jp->to->server, jp->from, yahoodata);

            /* notify user of the security hole */
            message = xmlnode_new_tag("message");
            body    = xmlnode_insert_tag(message, "body");
            subject = xmlnode_insert_tag(message, "subject");

            xmlnode_put_attrib(message, "to", jid_full(jp->from));
            xmlnode_put_attrib(message, "from", jid_full(jp->to));

            xmlnode_insert_cdata(subject, "SECURITY NOTICE", -1);
            xmlnode_insert_cdata(body, "Yahoo! Transport Security Notice:  You were logged in using a relatively insecure method, Your Yahoo! Password is being stored in Plain Text in Your roster.  If you would like your password to be stored using a more secure method, please un-register with the Yahoo! Transport, and re-register, so that Yahoo! Transport can safely store your password on the Server.\n\nNOTE: You will not receive this message again.", -1);

            deliver(dpacket_new(message), yjp->yti->i);
        }
        else
        {
            xmlnode err, m = xmlnode_new_tag("message");

            xmlnode_put_attrib(m, "type", "error");
            xmlnode_put_attrib(m, "from", jid_full(jp->to));
            xmlnode_put_attrib(m, "to", jid_full(jp->from));

            err = xmlnode_insert_tag(m, "error");
            xmlnode_put_attrib(err, "code", "407");
            xmlnode_insert_cdata(err, "No Logon Data Found", -1);

            deliver(dpacket_new(m), yjp->yti->i);
            xmlnode_free(jp->x);

            return;
        }
    }
    else
    {
        username = xmlnode_get_attrib(x, "id");
        password = xmlnode_get_attrib(x, "pass");
    }

    log_debug("yahoo","yahoo_auth_user: logging in session for %s", jid_full(jp->from));
    yahoo_session_create(yjp->yti, username, password, jp);

    if(x != NULL)
        xmlnode_free(x);

    return;
}


/* subscribe to a yahoo user */
int yahoo_subscribe_user(yahootrans_jpacket yjp)
{
    jpacket jp = yjp->jp;
    yahoo_session s;
        
    s = yahoo_find_session(yjp->yti, jp->from);

    if(s == NULL || s->Con == NULL) 
    {
        yahoo_send_error(yjp->yti, jp->from, "407", "Server failed to find your session--cannot subscribe to yahoo user without a session");
        xmlnode_free(jp->x);
        return 1;
    }

    log_debug("yahoo", "User %s subscribing to yahoo user %s", jid_full(jp->from), jp->to->user);
    
    if(jp->to->user == NULL)
    { /* subscribing to the transport */
        yahoo_send_presence(s, jp, jid_full(jp->to), "subscribed", "Transport Subscribe", NULL);
    }
    else
    { /* auth the s10n, and add the user to the yahoo roster */
        yahoo_send_presence(s, NULL, jid_full(jp->to), "subscribed", "Transport User Subscribe", NULL);
        log_debug("yahoo", "Subscriptions sent, adding buddy %s", jp->to->user);
        yahoo_add_buddy(s->Con, jp->to->user, s->Con->login_id, "jabber_yt", "");
        yahoo_send_presence(s, jp, jid_full(jp->to), "subscribe", "Transport User Subscribe", NULL);
    }

    return 0;
}

/* unsubscribe from a yahoo user */
int yahoo_unsubscribe_user(yahootrans_jpacket yjp)
{
    jpacket jp = yjp->jp;
    yahoo_session s;

    s = yahoo_find_session(yjp->yti, jp->from);

    if(s == NULL || jp->to->user == NULL || s->Con == NULL || s->Con->login_id == NULL) 
    {
        yahoo_send_error(yjp->yti, jp->from, "407", "Server failed to find your session");
        xmlnode_free(jp->x);
        return 1;
    }


    log_debug("yahoo", "[unsub] User %s unsub to %s", jid_full(jp->from), jp->to->user);
    yahoo_remove_buddy(s->Con, jp->to->user, s->Con->login_id, "jabber_yt", "");
    yahoo_send_presence(s, jp, jid_full(jp->to), "unsubscribe", "Transport User Unsubscribe", NULL);

    return 0;
}

/* updated the status for the session based on the presece xmlnode x */
void yahoo_update_status(yahootrans_instance yti, yahoo_session s, xmlnode x)
{
    pool p;
    char *show = NULL;
    xmlnode xshow;

    if(s == NULL || x == NULL || s->Con == NULL) 
    {
        if(x != NULL) xmlnode_free(x);
        return;
    }

    p = xmlnode_pool(x);

    xshow = xmlnode_get_tag(x, "show");
    if(xshow != NULL)
        show = pstrdup(p, xmlnode_get_data(xshow));

    log_debug("yahoo", "[auth] (%s) switching status (%d) -> (%s)", jid_full(s->user), s->status,show);

    if(show == NULL || j_strcmp(show, "chat") == 0)
    {
        if(s->status != YAHOO_STATUS_AVAILABLE)
        yahoo_cmd_set_back_mode(s->Con, YAHOO_STATUS_AVAILABLE, xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(x, "status"))));
        s->status = YAHOO_STATUS_AVAILABLE;
    }
    else if(j_strcmp(show, "away") == 0)
    {
        yahoo_cmd_set_away_mode(s->Con, YAHOO_STATUS_BRB, xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(x, "status"))));
        s->status = YAHOO_STATUS_BRB;
    }
    else if(j_strcmp(show, "xa") == 0)
    {
        yahoo_cmd_set_away_mode(s->Con, YAHOO_STATUS_IDLE, xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(x, "status"))));
        s->status = YAHOO_STATUS_IDLE;
    }
    else if(j_strcmp(show, "dnd") == 0)
    {
        yahoo_cmd_set_away_mode(s->Con, YAHOO_STATUS_BUSY, xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(x, "status"))));
        s->status = YAHOO_STATUS_BUSY;
    }

    jutil_tofrom(x);
    deliver(dpacket_new(x), yti->i);
}

/* parse call from parser.c */
int yahoo_parse_presence(yahootrans_jpacket yjp)
{
    jpacket jp = yjp->jp;
    xmlnode temp;
    yahoo_session s;
    char *res;

    switch(jpacket_subtype(jp))
    {
    case JPACKET__UNAVAILABLE:
        if(jp->to->resource == NULL || jp->to->user != NULL)
        { /* presence to a user */
            jutil_tofrom(jp->x);
            deliver(dpacket_new(jp->x), yjp->yti->i);
            break;
        }

        yjp->yti->PPDB = ppdb_insert(yjp->yti->PPDB, jp->from, jp->x);

        s = yahoo_find_session(yjp->yti, jp->from);

        res = jp->from->resource;
        jid_set(jp->from, NULL, JID_RESOURCE);

        temp = xmlnode_dup(ppdb_primary(yjp->yti->PPDB, jp->from));
        jid_set(jp->from, res, JID_RESOURCE);

        if(temp != NULL) /*session is still active, update status*/
        {
            log_debug("yahoo", "Received unavailable, but session is still active, updating status");
            yahoo_update_status(yjp->yti, s, temp);
        }
        else /*all sessions closed, exit session*/
        {
            if(s != NULL) 
            {
                /* send presence from all users */
                while(s->online_users->next != NULL)
                {
                    xmlnode x = xmlnode_dup(jp->x);
                    jutil_tofrom(x);
                    deliver(dpacket_new(x), yjp->yti->i);
                    s->online_users = s->online_users->next;
                }
                pool_free(s->online_users->p);
                s->online_users = NULL;
                yahoo_session_end(s);
            }

            /* mirror the unavail presence */
            jutil_tofrom(jp->x);
            deliver(dpacket_new(jp->x), yjp->yti->i);
        }
        break;
    case JPACKET__AVAILABLE:
        if(jp->to->resource == NULL || jp->to->user != NULL)
        { /* presence to a user */


            /* avail to a user, add it to our user list */
            s = yahoo_find_session(yjp->yti, jp->from);

            if(s == NULL)
            { /* we don't have a session yet -- XXX queue */
                xmlnode_free(jp->x);
                break;
            }

            if(s->online_users == NULL)
            {
                pool p = pool_new();
                jid j = jid_new(p, jid_full(jp->to));
                s->online_users = j;
            }
            else
            {
                jid_append(s->online_users, jp->to);
            }
            break;
        }

        yjp->yti->PPDB = ppdb_insert(yjp->yti->PPDB, jp->from, jp->x);
        yahoo_auth_user(yjp);

        break;
    default:
        xmlnode_free(jp->x);
        break;
    };
    return 0;
}

void yahoo_send_presence(yahoo_session s, jpacket jp, char *from, char *type, char *status, char *show)
{
    xmlnode presence;
    jid j;

    if(s == NULL || from == NULL) 
        return;

    presence = xmlnode_new_tag("presence");
    xmlnode_put_attrib(presence,"to",jid_full(s->user));
    xmlnode_put_attrib(presence,"from",from);

    if(type != NULL)
    {
        xmlnode_put_attrib(presence,"type",type);
    }
    else /* if it's an available pres, insert it into online_users */
    {
        /* insert it into the presence list */
        j = jid_new(xmlnode_pool(presence), from);
        if(s->online_users == NULL)
        {
            pool p = pool_new();
            jid temp = jid_new(p, jid_full(j));
            s->online_users = temp;
        }
        else
        {
            jid_append(s->online_users, j);
        }
    }

    if(status != NULL)
        xmlnode_insert_cdata(xmlnode_insert_tag(presence, "status"), status, -1);

    if(show != NULL)
        xmlnode_insert_cdata(xmlnode_insert_tag(presence, "show"), show, -1);

    deliver(dpacket_new(presence), s->yti->i);

    if(jp != NULL) 
        xmlnode_free(jp->x);
}
