/*
Yahoo-Transport sessions.c Handles the main session loop
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include <yahootrans.h>
#define YAHOO_PAGER_HOST "cs.yahoo.com"
#define YAHOO_PAGER_PORT 5050

struct yahoo_options Opts={0,NULL,0};

void _yahoo_session_end(void *arg)
{
    yahoo_session s = (yahoo_session)arg;
    log_debug("yahoo", "FINAL: ending yahoo session for %s", jid_full(s->user));
    if(s->Con != NULL)
        yahoo_free_context(s->Con);
    s->Con = NULL;
    pool_free(s->p);
}

/* exit the session */
void yahoo_session_end(yahoo_session s)
{
    if(s != NULL)
    {
        log_debug("yahoo", "Session (%s) told to end.", jid_full(s->user));
        yahoo_send_presence(s, NULL, jid_full(s->id), "unavailable", "Yahoo! Offline", NULL);
        if(s->state == YAHOO_STATE_ONLINE) 
        {
            char *res = s->user->resource;
            s->state = YAHOO_STATE_ERROR;
            jid_set(s->user, NULL, JID_RESOURCE);
            xhash_zap(s->yti->s, jid_full(s->user));
            jid_set(s->user, res, JID_RESOURCE);
            log_notice(s->yahoo_host, "Closing down session for %s", jid_full(s->user));
        }
        log_debug("yahoo", "closing mio %X", s->m);
        mio_close(s->m);
    }
}

/* status packet from a yahoo user to a jabber user */
void process_packet_status(struct yahoo_packet *pkt,yahoo_session s)
{
    int a;

    if(s->Con == NULL) 
        return;

    if(s == NULL || pkt == NULL || s->Con->user == NULL) 
        return;
    
    if (pkt->service == YAHOO_SERVICE_LOGOFF && pkt->msgtype == 0xFFFFFFFF)
    {
        yahoo_send_error(s->yti, s->user,"502", "Error from Yahoo! server, You have been disconnected, Send Online presence again to re-log in.");
        yahoo_session_end(s);
        return;
    }

    for(a = 0; a < pkt->idstatus_count; a++)
    {
        struct yahoo_idstatus *rec;
        spool id;
        pool p;
        
        p = pool_new();
        id = spool_new(p);

        rec = pkt->idstatus[a];
        spooler(id,rec->id,"@",s->yahoo_host,id);
        if(rec->in_pager)
        {
            if(pkt->service==YAHOO_SERVICE_ISAWAY)
            {
                switch(rec->status)
                {
                case YAHOO_STATUS_ONPHONE:
                case YAHOO_STATUS_IDLE:
                case YAHOO_STATUS_STEPPEDOUT:
                case YAHOO_STATUS_OUTTOLUNCH:
                case YAHOO_STATUS_BRB:
                    yahoo_send_presence(s,NULL,spool_print(id),NULL,yahoo_get_status_string(rec->status),"away");
                    break;
                case YAHOO_STATUS_NOTATDESK:
                case YAHOO_STATUS_NOTINOFFICE:
                case YAHOO_STATUS_ONVACATION:
                case YAHOO_STATUS_NOTATHOME:
                    yahoo_send_presence(s,NULL,spool_print(id),NULL,yahoo_get_status_string(rec->status),"xa");
                    break;
                case YAHOO_STATUS_BUSY:
                    yahoo_send_presence(s,NULL,spool_print(id),NULL,yahoo_get_status_string(rec->status),"dnd");
                    break;
                default:
                    yahoo_send_presence(s,NULL,spool_print(id),NULL,"Yahoo! Online",NULL);
                    break;
                }
                
            }
            else
            {
                yahoo_send_presence(s,NULL,spool_print(id),NULL,"Yahoo! Online",NULL);
            }
        }
        else
        {
            yahoo_send_presence(s,NULL,spool_print(id),"unavailable","Yahoo! user went offline",NULL);
        }
        pool_free(p);
    }
}

/* handles a message from yahoo to jabber */
void process_packet_message(pool p,struct yahoo_packet *pkt, yahoo_session s)
{

    if(pkt->msgtype == YAHOO_MSGTYPE_STATUS)
    {
        spool id=spool_new(p);

        spooler(id,pkt->msg_id,"@",s->yahoo_host,id);

        yahoo_send_presence(s,NULL,spool_print(id),NULL,yahoo_get_status_string(pkt->msg_status),NULL);
        return;
    }


    if(pkt->msg)
    {
        spool id=spool_new(p);
        spooler(id,pkt->msg_id,"@",s->yahoo_host,id);
        if(pkt->msgtype==YAHOO_MSGTYPE_BOUNCE) 
            yahoo_send_message(s,NULL,spool_print(id),NULL,"Message not sent: user went offline",NULL);
        else
        {
            char *msg=str_to_UTF8(p,pkt->msg);
            yahoo_send_message(s,NULL,spool_print(id),"chat",msg,NULL);
        }
    }
}


/* handles a message from yahoo to jabber */
void process_packet_newmail(pool p,struct yahoo_packet *pkt, yahoo_session s)
{
    xmlnode message, x;

    if(pkt->mail_status)
    {
        char *number = pmalloco(p, 4);
        snprintf(number, 3, "%d", pkt->mail_status);

        message = xmlnode_new_tag("message");
        xmlnode_put_attrib(message, "type", "headline");
        xmlnode_put_attrib(message, "to", jid_full(s->user));
        xmlnode_put_attrib(message, "from", s->yahoo_host);
        xmlnode_insert_cdata(xmlnode_insert_tag(message, "subject"), "Yahoo! E-mail notification", -1);
        xmlnode_insert_cdata(xmlnode_insert_tag(message, "body"), spools(p, "You have ", number, " new E-mail message(s) waiting", p), -1);
        x = xmlnode_insert_tag(message, "x");
        xmlnode_put_attrib(x, "xmlns", "jabber:x:oob");
        xmlnode_insert_cdata(xmlnode_insert_tag(x, "url"), "http://mail.yahoo.com", -1);
        xmlnode_insert_cdata(xmlnode_insert_tag(x, "desc"), "Check your Yahoo Email", -1);
        deliver(dpacket_new(message), s->yti->i);
    }
}
/* handles (sortof) a filetrnasfer request from yahoo to jabber */
void process_packet_fileXfer(struct yahoo_packet *pkt, yahoo_session s)
{
    pool   p = pool_new();
    spool sp = spool_new(p);
    spool sq = spool_new(p);;

    spooler(sp,pkt->file_from,"@",s->yahoo_host,sp);
    spooler(sq,"The Yahoo user, ",pkt->file_from,
        "would like to send you a file, due to limitations on yahoo-t, please point your browser to: ",
        pkt->file_url," for this file, Description follows: ",pkt->file_description,sq);

    yahoo_send_message(s,NULL,spool_print(sp),NULL,spool_print(sq),NULL);
    pool_free(p);
}

void handle_read(mio m, int flags, void *arg, char *buffer, int bufsz)
{
    struct yahoo_rawpacket *rawpkt;
    struct yahoo_packet *pkt;
    yahoo_session s = (yahoo_session)arg;
    pool p;

    if(flags == MIO_NEW || flags == MIO_ERROR)
        return;

    if(flags == MIO_CLOSED)
    {
        log_debug("yahoo", "KILLING: yahoo session for %s", jid_full(s->user));
        yahoo_session_end(s);
        mtq_send(s->q, s->p, _yahoo_session_end, (void*)s);
        return;
    }
    
    if(s == NULL || s->Con == NULL || s->state == YAHOO_STATE_ERROR) 
        return;

    if(s->Con == NULL) 
        return;

    p = pool_new();

    yahoo_addtobuffer(s->Con, buffer, bufsz);
    if(s->Con->io_buf_curlen > 102)
    {
        while(s->Con != NULL && (rawpkt = yahoo_getpacket(s->Con)))
        {
            pkt = yahoo_parsepacket(s->Con, rawpkt);
            log_debug("yahoo","Received packet:");
            log_debug("yahoo","Service = %s",yahoo_get_service_string(pkt->service));
            log_debug("yahoo","Real ID = %s",pkt->real_id);
            log_debug("yahoo","Active ID = %s",pkt->active_id);
            log_debug("yahoo","Connection ID = %X",pkt->connection_id);
            log_debug("yahoo","Magic ID = %X",pkt->magic_id);
            log_debug("yahoo","Unknown Flag 1 = %X",pkt->unknown1);
            log_debug("yahoo","Message Type = %X",pkt->msgtype);
            log_debug("yahoo","Raw Content = %s",rawpkt->content);
            switch(pkt->service)
            {
            case YAHOO_SERVICE_USERSTAT:
            case YAHOO_SERVICE_CHATLOGON:
            case YAHOO_SERVICE_CHATLOGOFF:
            case YAHOO_SERVICE_LOGON:
            case YAHOO_SERVICE_LOGOFF:
            case YAHOO_SERVICE_ISAWAY:
            case YAHOO_SERVICE_ISBACK:
                process_packet_status(pkt,s);
                break;
            case YAHOO_SERVICE_MESSAGE:
            case YAHOO_SERVICE_CHATMSG:
            case YAHOO_SERVICE_SYSMESSAGE:
                process_packet_message(p,pkt,s);
                break;
            case YAHOO_SERVICE_CHATINVITE:
                break;
            case YAHOO_SERVICE_NEWMAIL:
            case YAHOO_SERVICE_NEWPERSONALMAIL:
                process_packet_newmail(p,pkt,s);
                break;
            case YAHOO_SERVICE_NEWCONTACT:
                if(pkt->msg)
                    process_packet_message(p,pkt,s);
                else
                    process_packet_status(pkt,s);
                break;
            case YAHOO_SERVICE_GROUPRENAME:
                yahoo_get_config(s->Con);
                break;
            case YAHOO_SERVICE_CONFINVITE:
            case YAHOO_SERVICE_CONFLOGON:
            case YAHOO_SERVICE_CONFLOGOFF:
            case YAHOO_SERVICE_CONFMSG:
               break;
            case YAHOO_SERVICE_PING:
//               process_packet_ping(pkt,s);
               break;
            case YAHOO_SERVICE_FILETRANSFER:
               process_packet_fileXfer(pkt,s);
               break;
            }
            yahoo_free_packet(pkt);
            yahoo_free_rawpacket(rawpkt);
        }
    }

    pool_free(p);
}

/* create the session data, and start the main loop */
mio yahoo_session_create(yahootrans_instance yti, char *user,char *pword,jpacket jp)
{
    pool p;
    mio m = NULL;
    int buddyn = 0;
    struct yahoo_buddy *buddy;
    yahoo_session new = NULL;
    struct yahoo_context *Con;
    jid j;
        
    if(user == NULL || pword == NULL || jp == NULL)
        return NULL;

    if((Con = yahoo_init(user, pword, &Opts)) == NULL) 
    {
        log_notice(jp->to->server,"Session failed to start for %s as user %s -- bad username or password", jid_full(jp->from), user);

        return NULL;
    }

    if(!yahoo_connect(Con) || !yahoo_get_config(Con))
    {
        log_notice(jp->to->server,"Session failed to start for %s as user %s -- could not connect to yahoo or get config", jid_full(jp->from), user);
        yahoo_free_context(Con);
        return NULL;
    }

    if(yahoo_cmd_logon(Con, YAHOO_STATUS_AVAILABLE)) 
    {
        log_notice(jp->to->server,"Session failed to start for %s as user %s -- could not log in, probably bad password", jid_full(jp->from), user);
        yahoo_free_context(Con);
        return NULL;
    }

    if(Con->sockfd > 0)
        m = mio_new(Con->sockfd, &handle_read, NULL, mio_handlers_new(NULL, NULL, NULL)); 

    if(m == NULL) 
    {
        log_notice(jp->to->server,"Session failed to start for %s as user %s -- could not generate MIO object from yahoo fd", jid_full(jp->from), user);
        yahoo_free_context(Con);
        return NULL;
    }

    log_notice(jp->to->server,"Session started for %s as user %s", jid_full(jp->from), user);

    p           = pool_new();
    new         = pmalloco(p, sizeof(_yahoo_session));
    new->p      = p;
    new->yti    = yti;
    new->q      = mtq_new(p);
    new->user   = jid_new(new->p,jid_full(jp->from));
    new->Con    = Con;
    new->status = YAHOO_STATUS_AVAILABLE;
    new->state  = YAHOO_STATE_ONLINE;
    new->m      = m;

    mio_reset(m, &handle_read, (void*)new);

    if(jp != NULL)
    {
        new->yahoo_host = pstrdup(new->p, jp->to->server);
        new->id = jid_new(new->p,jid_full(jp->to));
        if(new->id->resource == NULL)
            jid_set(new->id, "registered", JID_RESOURCE);
    }
    else
    {
        new->yahoo_host = pstrdup(new->p, yti->i->id);
        new->id = jid_new(new->p, spools(new->p, yti->i->id, "/registered", new->p));
    }
    yahoo_send_presence(new, NULL, jid_full(new->id), "subscribe", "Setting Subscription", NULL);
    yahoo_send_presence(new, NULL, jid_full(new->id), NULL, "Yahoo! Online", NULL);

    p = pool_new();
    buddy = Con->buddies ? buddy = Con->buddies[buddyn] : NULL;
    while(buddy != NULL)
    {
        yahoo_send_presence(new, NULL, spools(p, buddy->id, "@", new->yahoo_host, p), "subscribe", "Old Yahoo! roster item", NULL);
        buddy = Con->buddies[++buddyn];
    }
    pool_free(p);

    j = jid_new(new->p, jid_full(new->user));
    jid_set(j, NULL, JID_RESOURCE);

    xhash_put(yti->s, jid_full(j), new);

    return m;
}
